import React, { useReducer, useRef, useEffect } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import { Icon } from '@arterial/icon';
import { List, ListItem, ListItemGraphic, ListItemText, ListItemPrimaryText, ListItemSecondaryText, ListItemMeta } from '@arterial/list';
import { MenuSurface, Corner } from '@arterial/menu-surface';
import { NotchedOutline } from '@arterial/notched-outline';
import { v4 } from 'uuid';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function HelperText(_ref) {
  var className = _ref.className,
      id = _ref.id,
      persistent = _ref.persistent,
      text = _ref.text,
      validationMsg = _ref.validationMsg,
      otherProps = _objectWithoutProperties(_ref, ["className", "id", "persistent", "text", "validationMsg"]);

  var classes = classNames('mdc-select-helper-text', className, {
    'mdc-select-helper-text--persistent': persistent,
    'mdc-select-helper-text--validation-msg': validationMsg
  });
  return /*#__PURE__*/React.createElement("p", Object.assign({
    className: classes,
    "aria-hidden": "true",
    id: id
  }, otherProps), text);
}
HelperText.displayName = 'SelectHelperText';
HelperText.propTypes = {
  className: PropTypes.string,
  id: PropTypes.string,
  persistent: PropTypes.bool,
  text: PropTypes.node,
  validationMsg: PropTypes.bool
};

function HelperLine(_ref) {
  var text = _ref.text;

  if (typeof text === 'object') {
    if (text === null || /*#__PURE__*/React.isValidElement(text)) {
      return text;
    }

    return /*#__PURE__*/React.createElement(HelperText, text);
  }

  return null;
}
HelperLine.displayName = 'SelectHelperLine';
HelperLine.propTypes = {
  text: PropTypes.oneOfType([PropTypes.node, PropTypes.object])
};

var _actions;

var INITIAL_STATE = {
  activated: false,
  focused: false,
  focusedIndex: 0,
  selectedIndex: -1,
  twoLine: false
};
var types = {
  ACTIVATE: 'ACTIVATE',
  DEACTIVATE: 'DEACTIVATE',
  FOCUS: 'FOCUS',
  FOCUS_INDEX: 'FOCUS_INDEX',
  SELECT_INDEX: 'SELECT_INDEX'
};
var actions = (_actions = {}, _defineProperty(_actions, types.ACTIVATE, function (state, action) {
  var focused = action.focused,
      focusedIndex = action.focusedIndex;
  return _objectSpread2(_objectSpread2(_objectSpread2({}, state), {}, {
    activated: true
  }, focused != null && {
    focused: focused
  }), focusedIndex != null && {
    focusedIndex: focusedIndex
  });
}), _defineProperty(_actions, types.DEACTIVATE, function (state, action) {
  var focused = action.focused;
  return _objectSpread2(_objectSpread2({}, state), {}, {
    activated: false
  }, focused != null && {
    focused: focused
  });
}), _defineProperty(_actions, types.FOCUS, function (state, action) {
  return _objectSpread2(_objectSpread2({}, state), {}, {
    focused: true
  });
}), _defineProperty(_actions, types.FOCUS_INDEX, function (state, action) {
  return _objectSpread2(_objectSpread2({}, state), {}, {
    focusedIndex: action.focusedIndex
  });
}), _defineProperty(_actions, types.SELECT_INDEX, function (state, action) {
  return _objectSpread2(_objectSpread2({}, state), {}, {
    activated: false,
    focusedIndex: action.selectedIndex,
    selectedIndex: action.selectedIndex
  });
}), _actions);
function reducer(state, action) {
  var applyAction = action.type ? actions[action.type] : null;
  if (!applyAction) throw new Error('Invalid action in reducer');
  return applyAction(state, action);
}

function Select(_ref) {
  var children = _ref.children,
      className = _ref.className,
      disabled = _ref.disabled,
      helperText = _ref.helperText,
      icon = _ref.icon,
      id = _ref.id,
      invalid = _ref.invalid,
      label = _ref.label,
      labelFloating = _ref.labelFloating,
      menuWidth = _ref.menuWidth,
      onSelect = _ref.onSelect,
      options = _ref.options,
      outlined = _ref.outlined,
      placeholder = _ref.placeholder,
      required = _ref.required,
      style = _ref.style,
      trailingIcon = _ref.trailingIcon,
      value = _ref.value,
      otherProps = _objectWithoutProperties(_ref, ["children", "className", "disabled", "helperText", "icon", "id", "invalid", "label", "labelFloating", "menuWidth", "onSelect", "options", "outlined", "placeholder", "required", "style", "trailingIcon", "value"]);

  var _useReducer = useReducer(reducer, INITIAL_STATE),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var anchorRef = useRef();
  var arterialRef = useRef(v4());
  var inputRef = useRef();
  var listItems = useRef(new Map());
  var classes = classNames('mdc-select', className, {
    'mdc-select--activated': state.activated,
    'mdc-select--disabled': disabled,
    'mdc-select--invalid': invalid,
    'mdc-select--focused': state.focused,
    'mdc-select--no-label': !label,
    'mdc-select--outlined': outlined,
    'mdc-select--required': required,
    'mdc-select--with-leading-icon': icon
  });
  var isLabelFloating = Boolean(labelFloating || state.focused || value);
  var labelClasses = classNames('mdc-floating-label', {
    'mdc-floating-label--float-above': isLabelFloating
  });
  var lineRippleClasses = classNames('mdc-line-ripple', {
    'mdc-line-ripple--active': state.focused
  });
  var labelId = "".concat(id, "-label");
  var ariaProps = {
    role: required ? null : 'button',
    'aria-haspopup': 'listbox',
    'aria-labelledby': label ? "".concat(labelId, " ").concat(id) : null,
    'aria-required': required
  };

  function focusListItem(index) {
    var i = index;

    if (!options[i] || options[i].disabled) {
      var next = options.slice(i + 1);
      i = next.findIndex(function (option) {
        return !option.disabled;
      });

      if (i === -1) {
        var previous = options.slice(0, i);
        i = previous.findIndex(function (option) {
          return !option.disabled;
        });
      }
    }

    if (i >= 0 && i < options.length) {
      setTimeout(function () {
        return listItems.current.get(i).focus();
      }, 1);
    }
  }

  function getListItemTabIndex(index) {
    if (options[index].disabled) return;
    return index === state.focusedIndex ? 0 : -1;
  }

  function getMenuStyle() {
    if (anchorRef.current) {
      var width = menuWidth || anchorRef.current.clientWidth;
      return {
        maxWidth: width,
        width: width
      };
    }
  }

  function getNextIndex(index) {
    var nextIndex = index + 1;
    var option = options[nextIndex];
    if (!option) return index;
    return option.disabled ? getNextIndex(nextIndex) : nextIndex;
  }

  function getPreviousIndex(index) {
    var previousIndex = index - 1;
    var option = options[previousIndex];
    if (!option) return index;
    return option.disabled ? getPreviousIndex(previousIndex) : previousIndex;
  }

  function getText(index) {
    var option = options[index] || {};
    return option.selectedText || option.text || '';
  }

  function handleClick(e) {
    if (disabled) return;
    if (!state.activated) focusListItem(state.selectedIndex);
    var type = state.activated ? types.DEACTIVATE : types.ACTIVATE;
    dispatch({
      type: type,
      focused: true
    });
    e.preventDefault();
  }

  function handleFocus() {
    dispatch({
      type: types.FOCUS
    });
  }

  function handleKeyDown(e) {
    var isEnter = e.key === 'Enter' || e.keyCode === 13;
    var isSpace = e.key === 'Space' || e.keyCode === 32;
    var arrowUp = e.key === 'ArrowUp' || e.keyCode === 38;
    var arrowDown = e.key === 'ArrowDown' || e.keyCode === 40;
    var isTab = e.key === 'Tab' || e.keyCode === 9;

    if (isEnter || isSpace || arrowUp || arrowDown) {
      e.preventDefault();
      focusListItem(state.selectedIndex);
      dispatch({
        type: types.ACTIVATE
      });
    } else if (isTab) {
      dispatch({
        type: types.DEACTIVATE,
        focused: false
      });
    }
  }

  function handleListItemAction(e, option, index) {
    var isClick = e.type === 'click';
    var isEnter = e.key === 'Enter' || e.keyCode === 13;
    var isSpace = e.key === 'Space' || e.keyCode === 32;
    var arrowUp = e.key === 'ArrowUp' || e.keyCode === 38;
    var arrowDown = e.key === 'ArrowDown' || e.keyCode === 40;
    var isTab = e.key === 'Tab' || e.keyCode === 9;

    if (!option.disabled && (isClick || isEnter || isSpace)) {
      e.preventDefault();
      setTimeout(function () {
        return anchorRef.current.focus();
      }, 0);
      dispatch({
        type: types.DEACTIVATE
      });
      if (onSelect) onSelect(option);
    } else if (isTab && e.shiftKey) {
      e.preventDefault();
      setTimeout(function () {
        return anchorRef.current.focus();
      }, 0);
    } else if (arrowUp) {
      e.preventDefault();
      var i = getPreviousIndex(index);
      focusListItem(i);
      dispatch({
        type: types.FOCUS_INDEX,
        focusedIndex: i
      });
    } else if (arrowDown) {
      e.preventDefault();

      var _i = getNextIndex(index);

      focusListItem(_i);
      dispatch({
        type: types.FOCUS_INDEX,
        focusedIndex: _i
      });
    }
  }

  useEffect(function () {
    function handleBodyClick(e) {
      var arterial = e.target.dataset.arterial;

      if (!arterial || arterial !== arterialRef.current) {
        dispatch({
          type: types.DEACTIVATE,
          focused: false
        });
      }
    }

    function handleWindowKeyDown(e) {
      var arterial = e.target.dataset.arterial;
      var isEscape = e.key === 'Escape' || e.keyCode === 27;
      var isTab = e.key === 'Tab' || e.keyCode === 9;

      if ((isEscape || isTab) && arterial === arterialRef.current) {
        if (isTab && !e.shiftKey) anchorRef.current.focus();
        dispatch({
          type: types.DEACTIVATE,
          focused: false
        });
      }
    }

    document.body.addEventListener('click', handleBodyClick);
    window.addEventListener('keydown', handleWindowKeyDown);
    return function () {
      document.body.removeEventListener('click', handleBodyClick);
      window.removeEventListener('keydown', handleWindowKeyDown);
    };
  }, []);
  useEffect(function () {
    if (value != null) {
      var selectedIndex = options.findIndex(function (option) {
        return option.value === value;
      });
      dispatch({
        type: types.SELECT_INDEX,
        selectedIndex: selectedIndex
      });
    }
  }, [options, value]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: classes,
    style: style
  }, /*#__PURE__*/React.createElement("div", Object.assign({}, ariaProps, otherProps, {
    className: "mdc-select__anchor",
    "data-arterial": arterialRef.current,
    onClick: handleClick,
    onFocus: handleFocus,
    onKeyDown: handleKeyDown,
    ref: anchorRef,
    style: {
      width: 'inherit'
    },
    tabIndex: disabled ? null : 0,
    title: getText(state.selectedIndex)
  }), !outlined && /*#__PURE__*/React.createElement("span", {
    className: "mdc-select__ripple"
  }), icon && /*#__PURE__*/React.createElement(Icon, {
    className: "mdc-select__icon",
    "data-arterial": arterialRef.current,
    icon: icon
  }), /*#__PURE__*/React.createElement("input", {
    className: "mdc-select__selected-text",
    "data-arterial": arterialRef.current,
    disabled: true,
    id: id,
    placeholder: placeholder,
    readOnly: true,
    ref: inputRef,
    value: getText(state.selectedIndex)
  }), trailingIcon ? /*#__PURE__*/React.createElement(Icon, {
    className: "mdc-select__dropdown-icon",
    "data-arterial": arterialRef.current,
    icon: trailingIcon,
    style: {
      background: 'initial'
    }
  }) : /*#__PURE__*/React.createElement("i", {
    className: "mdc-select__dropdown-icon",
    "data-arterial": arterialRef.current,
    id: "".concat(id, "-dropdown-icon")
  }), outlined ? /*#__PURE__*/React.createElement(NotchedOutline, {
    "data-arterial": arterialRef.current,
    label: label,
    labelId: labelId,
    notched: isLabelFloating
  }) : /*#__PURE__*/React.createElement(React.Fragment, null, label && /*#__PURE__*/React.createElement("span", {
    className: labelClasses,
    "data-arterial": arterialRef.current,
    id: labelId
  }, label), /*#__PURE__*/React.createElement("span", {
    className: lineRippleClasses,
    "data-arterial": arterialRef.current
  }))), /*#__PURE__*/React.createElement(HelperLine, {
    text: helperText
  }), /*#__PURE__*/React.createElement(MenuSurface, {
    className: "mdc-select__menu mdc-menu",
    anchorCorner: Corner.BOTTOM_LEFT,
    anchorRef: anchorRef,
    "data-arterial": arterialRef.current,
    id: "".concat(id, "-menu"),
    open: state.activated,
    style: getMenuStyle()
  }, /*#__PURE__*/React.createElement(List, {
    "data-arterial": arterialRef.current,
    id: "".concat(id, "-list"),
    role: "listbox",
    twoLine: state.twoLine
  }, Array.isArray(options) && options.length > 0 && options.map(function (option, index) {
    return /*#__PURE__*/React.createElement(ListItem, {
      "aria-selected": index === state.selectedIndex && !option.disabled,
      "data-arterial": arterialRef.current,
      "data-value": option.value,
      disabled: option.disabled,
      id: "".concat(id, "-list-item-").concat(option.value),
      key: option.value,
      onClick: function onClick(e) {
        return handleListItemAction(e, option, index);
      },
      onKeyDown: function onKeyDown(e) {
        return handleListItemAction(e, option, index);
      },
      ref: function ref(element) {
        return listItems.current.set(index, element);
      },
      role: "option",
      selected: index === state.selectedIndex && !option.disabled,
      tabIndex: getListItemTabIndex(index)
    }, option.graphic && /*#__PURE__*/React.createElement(ListItemGraphic, {
      "data-arterial": arterialRef.current,
      graphic: option.graphic
    }), state.twoLine ? /*#__PURE__*/React.createElement(ListItemText, {
      "data-arterial": arterialRef.current
    }, /*#__PURE__*/React.createElement(ListItemPrimaryText, {
      title: option.text
    }, option.text), /*#__PURE__*/React.createElement(ListItemSecondaryText, {
      title: option.secondaryText
    }, option.secondaryText), ")") : /*#__PURE__*/React.createElement(ListItemText, {
      "data-arterial": arterialRef.current,
      title: getText(index)
    }, option.text), option.node, option.meta && /*#__PURE__*/React.createElement(ListItemMeta, {
      "data-arterial": arterialRef.current,
      meta: option.meta
    }));
  })))));
}
Select.displayName = 'Select';
Select.propTypes = {
  children: PropTypes.node,
  className: PropTypes.string,
  disabled: PropTypes.bool,
  helperText: PropTypes.node,
  icon: PropTypes.node,
  id: PropTypes.string,
  invalid: PropTypes.bool,
  label: PropTypes.node,
  labelFloating: PropTypes.bool,
  menuWidth: PropTypes.string,
  onSelect: PropTypes.func,
  options: PropTypes.arrayOf(PropTypes.shape({
    disabled: PropTypes.bool,
    graphic: PropTypes.node,
    meta: PropTypes.node,
    node: PropTypes.node,
    secondaryText: PropTypes.string,
    selectedText: PropTypes.string,
    text: PropTypes.string.isRequired,
    value: PropTypes.string.isRequired
  })),
  outlined: PropTypes.bool,
  placeholder: PropTypes.string,
  required: PropTypes.bool,
  style: PropTypes.object,
  trailingIcon: PropTypes.node,
  value: PropTypes.string
};

export { HelperLine, HelperText, Select };
